@@ -14,19 +14,56 @@ use POSIX qw(WNOHANG);
 use Test::TCP qw(empty_port);
 use Time::HiRes qw(sleep);
 
-our $VERSION = '0.03';
+use constant PATH_SEP => $^O eq 'MSWin32' ? ';' : ':';
+
+our $VERSION = '0.04';
 
 our %Defaults = (
-    auto_start       => 1,
-    pid              => undef,
-    listen           => undef,
-    required_modules => [],
-    server_root      => undef,
-    tmpdir           => undef,
-    custom_conf      => '',
-    search_paths     => [ qw(/usr/sbin /usr/local/sbin /usr/local/apache/bin) ],
+    auto_start         => 1,
+    pid                => undef,
+    listen             => undef,
+    required_modules   => [],
+    server_root        => undef,
+    tmpdir             => undef,
+    custom_conf        => '',
+    search_paths       => [
+        qw(/usr/sbin /usr/local/sbin /usr/local/apache/bin)
+    ],
+    httpd              => 'httpd',
+    apxs               => 'apxs',
+    _fallback_dso_path => '',
 );
 
+if ($^O eq 'MSWin32') {
+    require Win32::Process;
+    Win32::Process->import;
+    my @cand_paths = map { $_ =~ s!/httpd\.exe$!!; $_ }
+        glob('C:/progra~1/apach*/apach*/bin/httpd.exe');
+    if (@cand_paths) {
+        # use the latest version, if any
+        my $path = $cand_paths[-1];
+        unshift @{$Defaults{search_paths}}, $path;
+        my $dso_path = $path;
+        $dso_path =~ s!/bin$!/modules!;
+        if (-d $dso_path) {
+            $Defaults{_fallback_dso_path} = $dso_path;
+        }
+    }
+} else {
+    # search for alternative names if necessary
+    my @paths = (
+        split(PATH_SEP, $ENV{PATH}),
+        @{$Defaults{search_paths}},
+    );
+    if (grep { -x "$_/$Defaults{httpd}" } @paths) {
+        # found
+    } elsif (grep { -x "$_/apache2" } @paths) {
+        # debian / ubuntu have these alternative names
+        $Defaults{httpd} = "apache2";
+        $Defaults{apxs} = "apxs2";
+    }
+}
+
 Class::Accessor::Lite->mk_accessors(keys %Defaults);
 
 sub new {
@@ -64,8 +101,8 @@ sub start {
         die "fork failed:$!";
     } elsif ($pid == 0) {
         # child process
-        $ENV{PATH} = join(':', $ENV{PATH}, @{$self->search_paths});
-        exec 'httpd', '-X', '-D', 'FOREGROUND', '-f', $self->conf_file;
+        $ENV{PATH} = join(PATH_SEP, $ENV{PATH}, @{$self->search_paths});
+        exec $self->httpd, '-X', '-D', 'FOREGROUND', '-f', $self->conf_file;
         die "failed to exec httpd:$!";
     }
     # wait until the port becomes available
@@ -79,9 +116,20 @@ sub start {
         ) and last;
         if (waitpid($pid, WNOHANG) == $pid) {
             die "httpd failed to start, exitted with rc=$?";
+            if (open my $fh, '<', "@{[$self->tmpdir]}/error_log") {
+                print STDERR do { local $/; join '', <$fh> };
+            }
         }
         sleep 0.1;
     }
+    # need to override pid on mswin32
+    if ($^O eq 'MSWin32') {
+        my $pidfile = "@{[$self->tmpdir]}/httpd.pid";
+        open my $fh, '<', $pidfile
+            or die "failed to open $pidfile:$!";
+        $pid = <$fh>;
+        chomp $pid;
+    };
     $self->pid($pid);
 }
 
@@ -89,8 +137,13 @@ sub stop {
     my $self = shift;
     die "httpd is not running"
         unless $self->pid;
-    kill 'TERM', $self->pid;
-    while (waitpid($self->pid, 0) != $self->pid) {
+    if ($^O eq 'MSWin32') {
+        Win32::Process::KillProcess($self->pid, 0);
+        sleep 1;
+    } else {
+        kill 'TERM', $self->pid;
+        while (waitpid($self->pid, 0) != $self->pid) {
+        }
     }
     $self->pid(undef);
 }
@@ -101,7 +154,12 @@ sub build_conf {
         my %static_mods = map { $_ => 1 } @{$self->get_static_modules};
         my %dynamic_mods = map { $_ => 1 } @{$self->get_dynamic_modules};
         my @mods_to_load;
+        my $httpd_ver = $self->get_httpd_version;
         for my $mod (@{$self->required_modules}) {
+            # rewrite authz_host => access for apache/2.0.x
+            if ($mod eq 'authz_host' && $self->get_httpd_version =~ m{2\.0\.}) {
+                $mod = 'access';
+            }
             if ($static_mods{$mod}) {
                 # no need to do anything
             } elsif ($dynamic_mods{$mod}) {
@@ -118,7 +176,9 @@ sub build_conf {
     my $conf = << "EOT";
 ServerRoot @{[$self->server_root]}
 PidFile @{[$self->tmpdir]}/httpd.pid
-LockFile @{[$self->tmpdir]}/httpd.lock
+<IfModule !mpm_winnt_module>
+  LockFile @{[$self->tmpdir]}/httpd.lock
+</IfModule>
 ErrorLog @{[$self->tmpdir]}/error_log
 Listen @{[$self->listen]}
 $load_modules
@@ -141,10 +201,22 @@ sub conf_file {
     return "@{[$self->tmpdir]}/httpd.conf";
 }
 
+sub get_httpd_version {
+    my $self = shift;
+    return $self->{_httpd_version} ||= do {
+        my $lines = $self->_read_cmd($self->httpd, '-v')
+            or die 'dying due to previous error';
+        $lines =~ m{Apache\/([0-9\.]+) }
+            or die q{failed to parse out version number from the output of "httpd -v"};
+        $1;
+    };
+}
+
 sub get_static_modules {
     my $self = shift;
     return $self->{_static_modules} ||= do {
-        my $lines = $self->_read_cmd('httpd', '-l');
+        my $lines = $self->_read_cmd($self->httpd, '-l')
+            or die 'dying due to previous error';
         my @mods;
         for my $line (split /\n/, $lines) {
             if ($line =~ /^\s+mod_(.*)\.c/) {
@@ -157,10 +229,27 @@ sub get_static_modules {
 
 sub get_dso_path {
     my $self = shift;
-    return undef
-        unless grep { $_ eq 'so' } @{$self->get_static_modules};
-    my $lines = $self->_read_cmd('apxs', '-q', 'LIBEXECDIR');
-    return (split /\n/, $lines)[0];
+    if (! exists $self->{_dso_path}) {
+        $self->{_dso_path} = sub {
+            return undef
+                unless grep { $_ eq 'so' } @{$self->get_static_modules};
+            # first obtain the path
+            my $path;
+            if (my $lines = $self->_read_cmd($self->apxs, '-q', 'LIBEXECDIR')) {
+                $path = (split /\n/, $lines)[0];
+            } elsif ($path = $self->_fallback_dso_path) {
+                warn "failed to obtain LIBEXECDIR from apxs, falling back to @{[$self->_fallback_dso_path]}";
+            } else {
+                die "failed to determine the apache modules directory";
+            }
+            # convert to shortname since SPs in path will let the glob fail
+            if ($^O eq 'MSWin32') {
+                $path = Win32::GetShortPathName($path);
+            }
+            return $path;
+        }->();
+    }
+    return $self->{_dso_path};
 }
 
 sub get_dynamic_modules {
@@ -180,20 +269,18 @@ sub get_dynamic_modules {
 sub _read_cmd {
     my ($self, @cmd) = @_;
     my ($rfh, $wfh);
-    my $pid = open2(
-        $rfh,
-        $wfh,
-        'env',
-        join(':', "PATH=$ENV{PATH}", @{$self->search_paths}),
-        @cmd,
-    ) or die "failed to run @{[join ' ', @cmd]}:$!";
+    local $ENV{PATH} = join PATH_SEP, $ENV{PATH}, @{$self->search_paths};
+    my $pid = open2($rfh, $wfh, @cmd)
+        or die "failed to run @{[join ' ', @cmd]}:$!";
     close $wfh;
     my $lines = do { local $/; join '', <$rfh> };
     close $rfh;
     while (waitpid($pid, 0) != $pid) {
     }
-    die "$cmd[0] exitted with a non-zero value:$?"
-        if $? != 0;
+    if ($? != 0) {
+        warn "$cmd[0] exitted with a non-zero value:$?";
+        return;
+    }
     return $lines;
 }
 
@@ -245,6 +332,8 @@ Application-specific configuration passed that will be written to the configurat
 
 An arrayref to specify the required apache modules.  If any module are specified, C<Test::Httpd::Apache2> will check the list of statically-compiled-in and dynamically-aviable modules and load the necessary modules automatically.  Module names should be specified excluding the "mod_" prefix and ".so" suffix.  For example, C<auth_basic_module> should be specified as "auth_basic".  Default is an empty arrayref.
 
+Note: "Authz_host" is automatically translated to "access" if the found httpd is Apache/2.0.x for compatibility.
+
 =head3 search_paths
 
 Paths to look for the httpd server in addition to the PATH environment variable.  The default is: /usr/sbin, /usr/local/sbin, /usr/local/apache/bin.
